<?php if (!defined('BASEPATH')) die('No direct script access allowed');

/**
 * fieldtype
 *
 * @package     MSM Site Select fieldtype module
 * @category    Modules
 * @author      Rein de Vries <info@reinos.nl>
 * @link        http://reinos.nl
 * @copyright   Copyright (c) 2013 Reinos.nl Internet Media
 */

include(PATH_THIRD.'msm_site_select/config.php');


class Msm_site_select_ft extends EE_Fieldtype
{

    public $info = array(
        'name'      => MSM_SITE_SELECT_NAME,
        'version'   => MSM_SITE_SELECT_VERSION
    );

    public $settings = array();

    public $default_settings = array();

    public $has_array_data = TRUE;

    private $prefix;

    // ----------------------------------------------------------------------

    /**
     * Constructor
     *
     * @access public
     *
     * Calls the parent constructor
     */
    public function __construct()
    {
        if (version_compare(APP_VER, '2.1.4', '>')) { parent::__construct(); } else { parent::EE_Fieldtype(); }

        $this->prefix = MSM_SITE_SELECT_MAP.'_';

        //load lang file
        ee()->lang->loadfile(MSM_SITE_SELECT_MAP);

        //do we need to update
        $this->update();

        //require settings
        require 'settings.php';
    }

    // ---------------------------------------------------------------------- 
    // Before saving the content to the database
    // ---------------------------------------------------------------------- 
    
    /**
     * save (Native EE)
     *
     * @access public
    */
    public function save($data)
    {
        return $this->_save($data);
    }

    // ---------------------------------------------------------------------- 

    /**
     * save (Low Variables)
     *
     * @access public
    */
    public function save_var_field($data)
    {
        return $this->_save($data);
    }

    // ---------------------------------------------------------------------- 

    /**
     * save (Matrix)
     *
     * @access public
    */
    public function save_cell($data)
    {
        return $this->_save($data);
    }

    // ---------------------------------------------------------------------- 

    /**
     * save (Content Elements)
     *
     * @access public
    */
    public function save_element($data)
    {
        return $this->_save($data);
    }

    // ---------------------------------------------------------------------- 

    /**
     * save
     *
     * @access public
    */
    private function _save($data = '')
    {
        if(is_array($data))
        {
            $data = implode('|', $data);
        }
        
        return $data;
    }

    // ----------------------------------------------------------------------
    // Display the field for all types
    // ----------------------------------------------------------------------

    /**
     * display_field
     *
     * @access public
    */
    function display_field($data)
    {
        //generate the data
        return $this->_display_field($data);
    }

    // ----------------------------------------------------------------------

    /**
     * display_var_field (Low variables)
     *
     * @access public
    */
    function display_var_field ($data)
    {
        //generate the data
        return $this->_display_field($data, 'low_variables');
    }

    // ----------------------------------------------------------------------
    
    /**
     * display_cell (MATRIX)
     *
     * @access public
    */
    function display_cell( $data )
    {
       return $this->_display_field($data, 'matrix');
    }

    // ----------------------------------------------------------------------
    
    /**
     * display_element (Content Elements)
     *
     * http://www.krea.com/docs/content-elements/element-development/ee2-functions-reference
     *
     * @access public
    */
    function display_element($data)
    {       
        return $this->_display_field($data, 'content_elements');
    }
    
    // ----------------------------------------------------------------------

    /**
     * display_field
     *
     * @access public
    */
    private function _display_field($data, $type = 'default')
    {
        //save data
        $data = trim($data);

        //fieldname
        switch($type)
        {
            case 'matrix': $field_name = $this->cell_name ;
                break;
            case 'default':
            default : $field_name = $this->field_name;
                break;
        }

        //get the sites
        ee()->db->select('*');
        ee()->db->from('sites');
        $result = ee()->db->get();
        
        $list = array(
            '0' => lang(MSM_SITE_SELECT_MAP.'_select_site')
        );
        
        //allowed
        $_allowed = $this->settings['msm_site_select_allowed_sites'];
        $allowed = array();
        if(!empty($_allowed))
        {
            foreach($_allowed as $v)
            {
                $allowed[$v] = $v;
            }   
        }
        
        if($result->num_rows() > 0)
        {
            foreach($result->result() as $row)
            {
                if(!empty($allowed) && isset($allowed[$row->site_id]))
                {
                    $list[$row->site_id] = $row->site_label;
                } 
                else if(empty($allowed))
                {
                    $list[$row->site_id] = $row->site_label;
                }
            }
        }
        
        //multiselect
        if($this->settings[$this->prefix.'multiselect'])
        {
            $data = explode('|', $data);
            
            return form_multiselect($field_name.'[]', $list, $data);
        }
        return form_dropdown($field_name, $list, $data);
    }
    
    // ----------------------------------------------------------------------
    // Replace the tags for all types
    // ----------------------------------------------------------------------

    /**
     * display_var_tag (Low variables)
     *
     * @access public
    */
    public function display_var_tag($var_data, $tagparams, $tagdata)
    {
        return $this->replace_tag($var_data, $tagparams, $tagdata);
    }

    // ----------------------------------------------------------------------

    /**
     * replace_element_tag (Content Elements)
     *
     * @access public
    */
    public function replace_element_tag($data, $params = array(), $tagdata)
    {
        return $this->replace_tag($data, $params, $tagdata);
    }

    // ----------------------------------------------------------------------

    /**
     * replace_tag
     *
     * @access public
    */
    public function replace_tag($data, $params = array(), $tagdata = FALSE)
    {
        $data = trim($data);
        if ($data == FALSE) return;

        //tmp
        $tmp = explode('|', $data);

        //select param
        if(isset($params['select']) && isset($tmp[$params['select']]))
        {
            return $tmp[$params['select']];
        }

        return $data;
    }

    // ----------------------------------------------------------------------
    
    /**
     * replace_tag_catchall
     *
     * @access public
    */
    /*function replace_tag_catchall($file_info, $params = array(), $tagdata = FALSE, $modifier)
    {
    
    }*/

    // ----------------------------------------------------------------------
    // Display the settings for all types
    // ----------------------------------------------------------------------
    
    /**
     * Display settings screen (Default EE)
     *
     * @access  public
     */
    function display_settings($data)
    {
        foreach($this->_display_settings($data) as $val)
        {
             ee()->table->add_row($val);
        }
    }

    // --------------------------------------------------------------------
    
    /**
     * Display settings screen (Matrix)
     *
     * @access  public
     */
    function display_cell_settings($data)
    {
        return $this->_display_settings($data, array('matrix_input' => 'class="matrix-textarea"'));
    }

    // --------------------------------------------------------------------
    
    /**
     * Display settings screen (Low variables)
     *
     * @access  public
     */
    function display_var_settings($data)
    {
        return $this->_display_settings($data);
    }

    // --------------------------------------------------------------------
    
    /**
     * Display settings screen (EE 2.7 GRID)
     *
     * @access  public
     */
    function grid_display_settings($data)
    {
        return $this->_display_settings($data, array(), 'grid');
    }

    // --------------------------------------------------------------------
    
    /**
     * Display settings screen (Content Elements)
     *
     * @access  public
     */
    function display_element_settings($data)
    {   
        return $this->_display_settings($data);
    }

    // --------------------------------------------------------------------

     /**
     * Display settings screen
     *
     * @access  public
     */
    private function _display_settings($data, $options = array(), $type = '')
    {
        $matrix_input = isset($options['matrix_input']) ? $options['matrix_input'] : '';

        $return = array();

        if(!empty($this->fieldtype_settings))
        {
            foreach($this->fieldtype_settings as $field=>$options)
            {
                switch($options['type'])
                {
                    //multiselect
                    case 'm' : 
                        //grid
                        if($type == 'grid')
                        {
                            $return[] = $this->grid_dropdown_row(
                                $options['label'],
                                $this->prefix.$options['name'].'[]',
                                $options['options'],
                                isset($data[$this->prefix.$options['name']]) ? $data[$this->prefix.$options['name']] : $options['def_value'],
                                true
                            );
                        }

                        //normal
                        else
                        {
                            $return[] = array(
                                $options['label'],
                                form_multiselect($this->prefix.$options['name'].'[]', $options['options'], (isset($data[$this->prefix.$options['name']]) ? $data[$this->prefix.$options['name']] : $options['def_value'] ))
                            );
                        }
                    break;

                    //select field
                    case 's' : 
                        //grid
                        if($type == 'grid')
                        {
                            $return[] = $this->grid_dropdown_row(
                                $options['label'],
                                $this->prefix.$options['name'],
                                $options['options'],
                                isset($data[$this->prefix.$options['name']]) ? $data[$this->prefix.$options['name']] : $options['def_value']
                            );
                        }

                        //normal
                        else
                        {
                            $return[] = array(
                                $options['label'],
                                form_dropdown($this->prefix.$options['name'], $options['options'], (isset($data[$this->prefix.$options['name']]) ? $data[$this->prefix.$options['name']] : $options['def_value'] ))
                            );
                        }
                        
                    break;

                    //text field
                    default:
                    case 't' : 
                        //grid
                        if($type == 'grid')
                        {
                           $return[] =  form_label($options['label']).NBS.NBS.NBS.
                            form_input(array(
                                'name'  => $this->prefix.$options['name'],
                                'value' => (isset($data[$this->prefix.$options['name']]) ? $data[$this->prefix.$options['name']] : $options['def_value'] ),
                                'class' => 'grid_input_text_small'
                            )).NBS.NBS.NBS;  
                        }

                        //normal
                        else
                        {
                            $return[] = array(
                                $options['label'],
                                form_input($this->prefix.$options['name'], (isset($data[$this->prefix.$options['name']]) ? $data[$this->prefix.$options['name']] : $options['def_value'] ), $matrix_input)
                            );
                        }
                    break;
                }
            }
        }

        return $return;
    }

    // ----------------------------------------------------------------------
    // Save the settings for all types
    // ----------------------------------------------------------------------
    
    /**
     * save_settings (Default EE)
     *
     * @access public
    */
    function save_settings($data)
    {  
        return $this->_save_settings($data);
    } 

    // --------------------------------------------------------------------
    
    /**
     * save_settings (matrix)
     *
     * @access public
    */
    function save_cell_settings($data)
    {  
        return $this->_save_settings($data, true);
    }  

    // --------------------------------------------------------------------
    
    /**
     * save_settings ((Low variables))
     *
     * @access public
    */
    function save_var_settings($data)
    {  
        return $this->_save_settings($data);
    }  

    // --------------------------------------------------------------------
    
    /**
     * save_settings (EE 2.7 GRID)
     *
     * @access public
    */
    function grid_save_settings($data)
    { 
        return $this->_save_settings($data, true);
    }  

    // --------------------------------------------------------------------

    /**
     * save_settings
     *
     * @access public
    */
    private function _save_settings($data, $use_data = false)
    { 
        $return = array();

        if(!empty($this->fieldtype_settings))
        {
            foreach($this->fieldtype_settings as $field=>$options)
            {
                $return[$this->prefix.$options['name']] =  $use_data ? $data[$this->prefix.$options['name']] : ee()->input->post($this->prefix.$options['name']);
            }
        }

        return $return;
    }  

    // ----------------------------------------------------------------------
    
    /**
     * install
     *
     * @access public
    */
    function install()
    {
        $return = array();

        if(!empty($this->fieldtype_settings))
        {
            foreach($this->fieldtype_settings as $field=>$options)
            {
                $return[$this->prefix.$options['name']] = $options['def_value'];
            }
        }

        return $return;
    }

    // ----------------------------------------------------------------------

    /**
     * Update
     */
    function update()
    {
        //get latest version
        $version = ee()->db->select('version, settings')->from('fieldtypes')->where('name', 'msm_site_select')->get()->row();
        
        if(isset($version->version))
        {
            $from = $version->version;

            if (! $from || $from == MSM_SITE_SELECT_VERSION) return FALSE;

           //update to version 1.1
            if (version_compare($from, '1.1', '<'))
            {
                $version->settings = unserialize(base64_decode($version->settings));
                $version->settings['allowed_sites'] = '';

                //update the settings
                ee()->db->where('name', 'msm_site_select');
                ee()->db->update('fieldtypes', array(
                    'settings' => base64_encode(serialize($version->settings)),
                    'version' =>  MSM_SITE_SELECT_VERSION
                ));
            }
        }

        return TRUE;
    }

    // ---------------------------------------------------------------------- 

    /**
     * accepts_content_type (GRID EE 2.7)
     *
     * @access public
    */
    public function accepts_content_type($name)
    {
        return ($name == 'channel' || $name == 'grid');
        //return true;
    }

    // --------------------------------------------------------------------

   

    // ---------------------------------------------------------------------- 
    // Custom 
    // ---------------------------------------------------------------------- 
       
    /**
     * _set_map_data
     *
     * @access private
    */
    public function get_allowed_sites()
    {
        //get the sites
        ee()->db->select('*');
        ee()->db->from('sites');
        $result = ee()->db->get();
        
        $list = array();
        
        if($result->num_rows() > 0)
        {
            foreach($result->result() as $row)
            {
                $list[$row->site_id] = $row->site_label;
            }
        }

        return $list;
    }

    // ----------------------------------------------------------------------
    
    /**
     * _set_params 
     *
     * @access private
    */
    /*private function _set_params($params)
    {
        $new_params = '';
        $method = 'geocoding'; //default
        $allowed_methods = array('geocoding', 'polygon', 'polyline'); //allowed

        if(!empty($params))
        {
            foreach($params as $key=>$val)
            {
                if($key == 'method')
                {
                    if(in_array($val, $allowed_methods))
                    {
                        $method = $val;
                    }
                }
                else
                {
                    $new_params .= $key.'="'.$val.'"';
                }
            }
        }

        return array('method' => $method, 'params' => $new_params);
    }*/

    // ----------------------------------------------------------------------

}

/* End of file ft.gmaps_fieldtype.php */
/* Location: ./system/expressionengine/third_party/gmaps_fieldtype/ft.gmaps_fieldtype.php */
